--[[
	Note to display developpers :
		In order to play nice with ShadowGreenLight you have to provide through the registerDisplay call at the end of file:
			- One object with the following methods and attributes set :
				* CreateDisplay() 					>>> Creating UI objects that you need
				* UpdateDisplay() 					>>> Update UI objects according to SGL state
				* UpdateDisplay_OnEnteringCombat()	>>> Update UI objects when entering combat
				* UpdateDisplay_OnExitingCombat()	>>> Update UI objects when exiting combat
				* options							>>> attribute containing an Ace3 option table for the user to configure your display							
			- One name (one word) that will identify your display
]]--

Display = LibStub("AceAddon-3.0"):NewAddon("ShadowGreenLight_DefaultDisplay", "AceConsole-3.0")
local ShadowGreenLight = ShadowGreenLight 	-- speed up lookup
local LibQTip = LibStub('LibQTip-1.0')		-- tooltip management library

Display.options = {
    reset = {
        type = "execute",
        handler = Display,
        name = "Reset position",
        desc = "Reset display position",
        func = "ResetDisplayPosition",
    },
	size = {
        type = "range",
        handler = Display,
        name = "Indicator size",
        desc = "Sets indicator display size",
        get = function(info)
            return info.handler.db.profile.indicatorSize
        end,
        set = function(info,v)
            info.handler.db.profile.indicatorSize = v
            info.handler:UpdateDisplaySettings()
        end,
        min = 4,
        max = 32,
        step = 1,
    },
	toggleBorder = {
        type = "toggle",
        handler = Display,
        name = "Indicator border toggle",
        desc = "Toggles displaying of indicator border",
        get = function(info)
            return info.handler.db.profile.displayIndicatorBorder
        end,
        set = function(info,v)
            info.handler.db.profile.displayIndicatorBorder = v
            info.handler:UpdateDisplaySettings()
            info.handler:Print("Indicator border " .. (v and "displayed" or "not displayed"))
        end,
    },
    toggleNumber = {
        type = "toggle",
        handler = Display,
        name = "Numeric display toggle",
        desc = "Toggles displaying of numbers",
        get = function(info)
            return Display.db.profile.displayNumericIndicator
        end,
        set = function(info,v)
            info.handler.db.profile.displayNumericIndicator = v
            info.handler:Print("Numeric indicator " .. (v and "shown" or "not shown"))
        end,
    },
    toggleTtp = {
        type = "toggle",
        handler = Display,
        name = "Time to profit display toggle",
        desc = "Toggles displaying of time to profit indicator",
        get = function(info)
            return info.handler.db.profile.displayTtpIndicator
        end,
        set = function(info,v)
            info.handler.db.profile.displayTtpIndicator = v
            info.handler:Print("Time to profit indicator " .. (v and "shown" or "not shown"))
        end,
    },
    toggleRecastIndicator = {
        type = "toggle",
        handler = Display,
        name = "Recast indicator toggle",
        desc = "Toggles displaying of recast indicator",
        get = function(info)
            return info.handler.db.profile.displayRecastIndicator
        end,
        set = function(info,v)
            info.handler.db.profile.displayRecastIndicator = v
            info.handler:Print("Recast indicator " .. (v and "shown" or "not shown"))
        end,
    },
    showTooltip = {
        type = "toggle",
        handler = Display,
        name = "Tooltip toggle",
        desc = "Toggles showing tooltip",
        get = function(info)
            return info.handler.db.profile.showTooltip
        end,
        set = function(info,v)
            info.handler.db.profile.showTooltip = v
            info.handler:Print("Tooltip " .. (v and "shown" or "not shown"))
        end,
    },
    toggleOOCHiding = {
        type = "toggle",
        handler = Display,
        name = "OoC hiding toggle",
        desc = "Toggles hiding indicator when out of combat",
        get = function(info)
            return Display.db.profile.oocHiding
        end,
        set = function(info,v)
            info.handler.db.profile.oocHiding = v
            info.handler:Print("ShadowGreenLight " .. (v and "hidden" or "shown") .. " out of combat")
            info.handler:UpdateDisplaySettings()
        end,
    },
}

function Display:OnInitialize()
  	-- Initialization of session db
  	self.db = LibStub("AceDB-3.0"):New("ShadowGreenLight_DefaultDisplayDB", {
  		profile = {
  			posX = 0,
  			posY = 0,
  			relativePoint = "CENTER",
  			point = "CENTER",
  			indicatorSize = 30,
  			displayIndicatorBorder = true,
  			displayNumericIndicator = true,
  			displayTtpIndicator = true,
  			displayRecastIndicator = true,
  			showTooltip = true,
  			oocHiding = false,
		}
	}, "Default")
end

function Display:OnEnable()
end

function Display:OnDisable()
end

--------------------------------------------------
-- Tooltip management

-- Font definition
local headerFont = CreateFont("ShadowGreenLight_HeaderFont")
headerFont:SetFont(GameTooltipText:GetFont(), 12)
headerFont:SetTextColor(0,1,1)

local whiteFont = CreateFont("ShadowGreenLight_WhiteFont")
whiteFont:SetFont(GameTooltipText:GetFont(), 10)
whiteFont:SetTextColor(1,1,1)

local redFont = CreateFont("ShadowGreenLight_RedFont")
redFont:SetFont(GameTooltipText:GetFont(), 10)
redFont:SetTextColor(1,0,0)

local orangeFont = CreateFont("ShadowGreenLight_OrangeFont")
orangeFont:SetFont(GameTooltipText:GetFont(), 10)
orangeFont:SetTextColor(1,0.5,0)

local greenFont = CreateFont("ShadowGreenLight_GreenFont")
greenFont:SetFont(GameTooltipText:GetFont(), 10)
greenFont:SetTextColor(0,1,0)

local yellowFont = CreateFont("ShadowGreenLight_YellowFont")
yellowFont:SetFont(GameTooltipText:GetFont(), 10)
yellowFont:SetTextColor(1,1,0)

local function showTooltip(self)
    if not Display.db.profile.showTooltip then return end
    -- Acquire tooltip
    local tooltip = LibQTip:Acquire("ShadowGreenLight_DefaultDisplay_Tooltip", 2, "LEFT", "RIGHT")
    Display.tooltip = tooltip 
    
    -- If we are in combat display tooltip
    if ShadowGreenLight.isInCombat then
        -- Add header
        tooltip:SetHeaderFont(headerFont)
        tooltip:AddHeader('Upgrade', 'Amount')
        
        -- Cycle the tooltip object
        for upgradeType, _ in pairs(ShadowGreenLight.currentTooltipUpgradeList) do
            tooltip:SetFont(whiteFont)
            tooltip:AddLine(upgradeType)
            for upgradeName, upgrade in pairs(ShadowGreenLight.currentTooltipUpgradeList[upgradeType]) do
                if upgrade.currentStack == 0 then
                    tooltip:SetFont(redFont)
                elseif upgrade.currentStack >= upgrade.maxStack then
                    tooltip:SetFont(greenFont)
                else
                    tooltip:SetFont(orangeFont)
                end
                tooltip:AddLine("  " .. upgradeName, upgrade.currentStack .. "/" .. upgrade.maxStack)
            end
        end
    else
        -- Add header
        tooltip:SetHeaderFont(headerFont)
        tooltip:AddHeader('ShadowGreenLight')
        -- Add hint
        tooltip:SetFont(yellowFont)
        tooltip:AddLine("Right click and drag to move")
    end
    
    -- Use smart anchoring code to anchor the tooltip to our frame
    tooltip:SmartAnchorTo(self)
    
    -- Show it, et voilà !
    tooltip:Show()
end

local function hideTooltip(self)
  if not Display.db.profile.showTooltip then return end
  -- Release the tooltip
  LibQTip:Release(Display.tooltip)
  Display.tooltip = nil
  
end

--------------------------------------------------
-- Create all necessary UI objects
function Display:CreateDisplay()
    if self.frame then return end
    -- Create fugly square
	self.frame = CreateFrame("Frame", "ShadowGreenLight_DefaultDisplay_Frame", UIParent)
	self.frame:SetPoint(self.db.profile.point,UIParent,self.db.profile.relativePoint,self.db.profile.posX,self.db.profile.posY)
	self.frame:EnableMouse()
	self.frame:SetMovable(true)
	self.frame:SetResizable(true)
	-- Mouse behaviour : moving
	self.frame:SetScript("OnMouseDown", function() 
		if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "RightButton" ) ) then
			this:StartMoving();
			this.isMoving = true;
		end
	end)
	self.frame:SetScript("OnMouseUp", function() 
		if ( this.isMoving ) then
			local point,relativeTo,relativePoint,xOfs,yOfs = this:GetPoint(1)
			Display.db.profile.point = point
			Display.db.profile.relativePoint = relativePoint
			Display.db.profile.posX = xOfs
			Display.db.profile.posY = yOfs
			this:StopMovingOrSizing();
			this.isMoving = false;
		end
	end)
	-- Mouse behaviour : tooltip
	self.frame:SetScript('OnEnter', showTooltip)
    self.frame:SetScript('OnLeave', hideTooltip)

	
	-- create recast indicator
	self.frame.recastIndicator = self.frame:CreateTexture()
    self.frame.recastIndicator:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcon_8")
    self.frame.recastIndicator:SetPoint("CENTER", self.frame, "CENTER", 0,0)
	self.frame.recastIndicator:Hide()
	
	-- create numeric indicator
	self.frame.numericIndicator = self.frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	self.frame.numericIndicator:SetPoint("BOTTOMLEFT", self.frame, "TOPLEFT", 0, 0)
	self.frame.numericIndicator:SetTextColor(1,1,1,1)
	self.frame.numericIndicator:SetTextHeight("12")
	self.frame.numericIndicator:Hide()
		
	-- create time to profit indicator
	self.frame.ttpIndicator = self.frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	self.frame.ttpIndicator:SetPoint("TOPLEFT", self.frame, "BOTTOMLEFT", 0, 0)
	self.frame.ttpIndicator:SetTextColor(1,1,1,1)
	self.frame.ttpIndicator:SetTextHeight("12")
	self.frame.ttpIndicator:Hide()
	
	-- update configuration-dependant attributes
	self:UpdateDisplaySettings()
end

---------------------------------------------------
-- Update Display according to ShadowGreenLight state
function Display:UpdateDisplay()
	-- Compute current upgrade
    local currentUpgrade = ShadowGreenLight:ComputeCurrentUpgrade()
		
	-- Change background color
	self.frame:SetBackdropColor(
		1-currentUpgrade / ShadowGreenLight.maxUpgrade,
		currentUpgrade / ShadowGreenLight.maxUpgrade,
		0,
		1
	)
	
	-- Display/Hide recast indicator according to available upgrade versus last time SWP was applied
	if self.db.profile.displayRecastIndicator and currentUpgrade > ShadowGreenLight.lastSWPUpgrade then
	   self.frame.recastIndicator:Show()	
	else
	   self.frame.recastIndicator:Hide()
	end
	
	-- Display/Hide time to profit indicator according to config and if available upgrade versus last time SWP was applied
	if self.db.profile.displayTtpIndicator and currentUpgrade > ShadowGreenLight.lastSWPUpgrade then
		local ttp = ShadowGreenLight:GetTimeToProfit(currentUpgrade)
		self.frame.ttpIndicator:SetText(ttp)
		self.frame.ttpIndicator:Show()
	else
		self.frame.ttpIndicator:Hide()
	end	
	
	-- Display/Hide numeric indicator according to configuration
	if self.db.profile.displayNumericIndicator then
	   self.frame.numericIndicator:SetFormattedText( "%.0f/%.0f" , currentUpgrade, ShadowGreenLight.maxUpgrade)
	   self.frame.numericIndicator:Show()
	end
end

---------------------------------------------------
-- Update Display when entering combat
function Display:UpdateDisplay_OnEnteringCombat()
	self.frame:Show()
end

---------------------------------------------------
-- Update Display when entering combat
function Display:UpdateDisplay_OnExitingCombat()
	if self.db.profile.oocHiding then
		self.frame:Hide()
	end	
	
	self.frame.recastIndicator:Hide()
	self.frame.numericIndicator:Hide()
	
	self.frame:SetBackdropColor(0,0,0,1)
end


--------------------------------------------------
-- Update UI objects according to settings
function Display:UpdateDisplaySettings()
    self.frame:SetHeight(self.db.profile.indicatorSize)
	self.frame:SetWidth(self.db.profile.indicatorSize)
	self.frame:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = (self.db.profile.displayIndicatorBorder and "Interface\\Tooltips\\UI-Tooltip-Border") or "",
		tile = true, tileSize = 32, edgeSize = 16,
		insets = { left = 4, right = 4, top = 4, bottom = 4 }
	})
	self.frame:SetBackdropColor(0,0,0,1)
		
	self.frame.recastIndicator:SetWidth(self.db.profile.indicatorSize/2)
	self.frame.recastIndicator:SetHeight(self.db.profile.indicatorSize/2)
	
	-- hide if configured
	if self.db.profile.oocHiding then
		self.frame:Hide()
	end
end

--------------------------------------------------
-- Reset UI objects position on the screen
function Display:ResetDisplayPosition()
	self.frame:ClearAllPoints()
	self.frame:SetPoint("CENTER",UIParent,"CENTER", 0, 0)
end

ShadowGreenLight:RegisterDisplay("Default", Display)